/***************************************************************************
                          kmatedit.h  -  description
                             -------------------
    begin                : Sam Mr 10 16:19:19 CET 2001
    copyright            : (C) 2001 by Rdiger Knrig
    email                : ruediger@knoerig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KMATEDIT_H
#define KMATEDIT_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <kapp.h>
#include <qwidget.h>
#include <qlabel.h>
#include <qspinbox.h>
#include <math.h>
#include <map>
#include "MainDlg.h"

/** enum which includes all lightsources in the order in which they are listened in the combobox */
enum lightsource_t {ambient,conic,directional,directional_patch,patch,point,soft,soft_conic,soft_directional};


/** enum which includes all lightsource attributes as flags */
enum lightattrib_t
{
	position = 0x1,
	patch_pos = 0x2,
	patch_sides = 0x4,
	direction = 0x8,
	spotattribs = 0x10,
	decay = 0x20,
	shadows = 0x40,
	cpattern = 0x80,
	softshadowattribs = 0x100
};

/** enum with all available effect filters */
enum sfxfilter_t {dof,film,glitter,halo};

/** Kmatedit is the base class of the project */
class Kmatedit : public MainDlg
{
  Q_OBJECT 
  public:
    /** construtor */
    Kmatedit(QWidget* parent=0, const char *name=0);
    /** destructor */
    ~Kmatedit();
public slots: // Public slots
  /** generates the text output */
  void generateText();
protected: // Protected methods
  /** generates the source code for a lightflow light source */
  QString makeLight();
  /** conversion degrees -> radians */
  float toRadians(int degrees) {return (degrees*M_PI/180.0f);}
  float toRadians(float degrees) {return (degrees*M_PI/180.0f);}
  /** initialise light tab enviroment */
  void prepareLight();
  /** generates the source code for a lightflow camera & saver enviroment */
  QString makeCamera();
  /**
  	* generates the code for an imager filter
		* @param filter - index of activated filter tab page
		*/
  QString makeCameraFilter(sfxfilter_t filter);
  /** generates the source code for a lightflow camera & saver enviroment */
  QString makeInterface();
  /** alter content of item according to user input */
  void alterOpListItem(QListViewItem *item);
  /** generates the python source code for a transformation stack  */
  QString makeTransformBlock();
protected slots: // Protected slots
  /**
  	* alters the explanation label right off the
		*	light source type selection box.
		* @param index index of lightsource selection box
		*/
  void generateLightSourceExplanation(int index) {LightSourceExplanation->setText(lightsourceexp[index]);}
  /** penumbra angle has to be at least conic angle */
  void coupleSpotAngles(int ang) {LightPenumbraAngle->setMinValue(ang);}
  /** disables all widgets in the light sheed 
			which are not required for the selected
			light source type */
  void lightDisableCheck(int );
  /** disables / enables widgets dependent on the selected 
widget type */
  void cameraDisableCheck(int );
  /** append entry to the end of list */
  void TransformListAppend();
  /** alter the current entry  */
  void TransformListAlter();
  /** No descriptions */
  void TransformListAdd();
  /** copy entry data to edit area and 
delete entry after */
  void TransformListCut();
  /** deletes the current entry */
  void TransformListDelete();
  /** No descriptions */
  void TransformListGet(QListViewItem *item);
protected: // Protected attributes
  /** contains an array of explanation strings for the light sources */
  static char *lightsourceexp[];
  /** contains an array with the names of all file-saving imagers */
  static char *imagesavers[];
  /** array with attribute flags describing used parameters  */
  lightattrib_t *lightsourceattribs;
  /** maps widgets to the light source attributes, which needs them */
  map<QWidget *,lightattrib_t> widgetallow;
};

#endif
